#!/bin/bash
#Automatically Created by buildpkg 1.6.1
#Maintainer: Cedric Durmont <cdurmont@gmail.com>
#url: http://nethack.org

######Begin Redundant Code######################################
check_for_root() {
	if [ "$UID" != "0" ]; then
		echo "You need to be root"
		exit 1
	fi
}

clean_dirs () {
        for COMPLETED in src pkg; do
                if [ -e $COMPLETED ]; then
                        rm -rf $COMPLETED
                fi
        done
}

clean_old_builds () {
	for COMPLETED in $package.{tgz,md5,dep}; do
		if [ "$KEEPDEPFILE" = "1" ] && [ $COMPLETED = $package.dep ]; then
			continue
		elif [ -e $COMPLETED ]; then
			rm -rf $COMPLETED
		fi
	done
	clean_dirs
}

set_pre_permissions() {
	cd $startdir/src
	find . -perm 664 -exec chmod 644 {} \;
	find . -perm 600 -exec chmod 644 {} \;
	find . -perm 444 -exec chmod 644 {} \;
	find . -perm 400 -exec chmod 644 {} \;
	find . -perm 440 -exec chmod 644 {} \;
	find . -perm 777 -exec chmod 755 {} \;
	find . -perm 775 -exec chmod 755 {} \;
	find . -perm 511 -exec chmod 755 {} \;
	find . -perm 711 -exec chmod 755 {} \;
	find . -perm 555 -exec chmod 755 {} \;
}


check_for_menu_compliance() {
	if [ -d "$startdir/pkg/usr/share/applications" ]; then
		cd $startdir/pkg/usr/share/applications
		for desktop in *.desktop; do
			Icon=$(grep -e "^Icon=" $desktop | sed 's/Icon=//')
			if [[ "$Icon" ]]; then
				if [ ! "$Icon" == "${Icon%.*}" ]; then
					sed '/Icon/s/\..*$//' -i $desktop
					Icon=$(grep -e "^Icon=" $desktop | sed 's/Icon=//')
				fi
				if [ -d "$startdir/pkg/usr/share/pixmaps" ]; then
					cd $startdir/pkg/usr/share/pixmaps
					for pixmaps in *; do
						[ -d $pixmaps ] && continue
						[ ! "${pixmaps##*.}" == "png" -a ! "${pixmaps##*.}" == "svg" ] && continue
				 		if [ "${pixmaps%.*}" == "$Icon" ]; then
				 			DIR=$(file $pixmaps | awk -F, '{print $2}' | sed 's/ //g')
							[ o"$DIR" == "o" ] && DIR="scalable"
							mkdir -p $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							mv $pixmaps $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							if [[ ! "$(ls $startdir/pkg/usr/share/pixmaps)" ]]; then
								rm -rf $startdir/pkg/usr/share/pixmaps
							fi
						fi
					done
				fi
				if [ -d "$startdir/pkg/usr/share/icons" ]; then
					cd $startdir/pkg/usr/share/icons
					for icons in *; do
				 		if [ "${icons%.*}" == "$Icon" ]; then
							[ ! "${pixmaps##*.}" == "png" -a ! "${pixmaps##*.}" == "svg" ] && continue
				 			DIR=$(file $icons | awk -F, '{print $2}' | sed 's/ //g')
							[ o"$DIR" == "o" ] && DIR="scalable"
							mkdir -p $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
							mv $icons $startdir/pkg/usr/share/icons/hicolor/$DIR/apps
						fi
					done
				fi
			fi
		done
	fi
}


gzip_man_and_info_pages() {
	for DOCS in man info; do
		if [ -d "$startdir/pkg/usr/share/$DOCS" ]; then
			mv $startdir/pkg/usr/share/$DOCS $startdir/pkg/usr/$DOCS
			if [[ ! "$(ls $startdir/pkg/usr/share)" ]]; then
				rm -rf $startdir/pkg/usr/share
			fi
		fi
		if [ -d "$startdir/pkg/usr/$DOCS" ]; then
			# I've never seen symlinks in info pages....
			if [ "$DOCS" == "man" ]; then
				(cd $startdir/pkg/usr/$DOCS
				for manpagedir in $(find . -type d -name "man*" 2> /dev/null) ; do
					( cd $manpagedir
					for eachpage in $( find . -type l -maxdepth 1 2> /dev/null) ; do
						ln -s $( readlink $eachpage ).gz $eachpage.gz
						rm $eachpage
					done )
				done)
			fi
			find $startdir/pkg/usr/$DOCS -type f -exec gzip -9 '{}' \;
		fi
	done
}

set_post_permissions() {
	for DIRS in usr/share/icons usr/doc; do
		if [ -d "$startdir/pkg/$DIRS" ]; then
			if [ "$DIRS" == "usr/doc" ]; then
				find $startdir/pkg/$DIRS -type f -exec chmod 644 {} \;
				find $startdir/pkg/$DIRS -type d -exec chmod 755 {} \;
			fi
		fi
		[ -d $startdir/pkg/$DIRS ] && chown root:root -R $startdir/pkg/$DIRS
	done
	[ -d $startdir/pkg/usr/bin ] && find $startdir/pkg/usr/bin -user root -group bin -exec chown root:root {} \;
}

copy_build_script() {
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver/
	cp $startdir/build-$pkgname.sh $startdir/pkg/usr/src/$pkgname-$pkgver/build-$pkgname.sh
	[ -f $startdir/ZENBUILD ] && cp $startdir/ZENBUILD $startdir/pkg/usr/src/$pkgname-$pkgver/ZENBUILD
}

create_package() {
	ls -lR $startdir/pkg
	cd $startdir/pkg
	/sbin/makepkg -l y -c n $startdir/$package.tgz
	cd $startdir
	md5sum $package.tgz > $startdir/$package.md5
}

strip_binaries() {
	cd $startdir/pkg
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | \
	xargs strip --strip-unneeded 2> /dev/null
}
#########End Redundant Code#####################################
#########Begin Non Redundant Code##############################

prepare_directory() {
	mkdir $startdir/src
	mkdir -p $startdir/pkg/usr/src/$pkgname-$pkgver
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
	        file=$(basename $SOURCES | awk -F= '{print $NF}')
		if [ ! -f "$file" ]; then
			if [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
				echo -e "\nDownloading $(basename $SOURCES)\n"
        	                wget $SOURCES -O $file
				if [ ! "$?" == "0" ]; then
					echo "Download failed"
					exit 2
				fi 
			else
				echo "$SOURCES does not appear to be a url nor is it in the directory"
				exit 2
			fi
		fi
		cp -R $file $startdir/src
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			cp -R $startdir/$(basename $SOURCES) $startdir/pkg/usr/src/$pkgname-$pkgver/
		fi
		if [ "${SOURCES##*.}" == "desktop" ]; then
			[[ "$(grep -e "^Categories=[a-zA-Z]*;" $(basename $SOURCES))" ]] && install -D $startdir/$(basename $SOURCES) /home/cedric/developpement/pkg_nethack/test/pkg/usr/share/applications/$(basename $SOURCES)
		fi
	done
}

extract_source() {
	cd $startdir/src
	if [[ "$(ls $startdir/src)" ]]; then	
		for FILES in ${source[@]}; do
	        	FILES="$(basename $FILES | awk -F= '{print $NF}')"
			file_type=$(file -biz "$FILES")
			unset cmd
			case "$file_type" in
				*application/x-tar*)
					cmd="tar -xf" ;;
				*application/x-zip*)
					cmd="unzip" ;;
				*application/x-gzip*)
					cmd="gunzip -d -f" ;;
				*application/x-bzip*)
					cmd="bunzip2 -f" ;;
			esac
			if [ "$cmd" != "" ]; then
				echo "$cmd $FILES"
	        	        $cmd $FILES
			fi
		done
	elif [ ! "$source" ]; then
		echo -n "" # lame fix
	else
		echo "no files in the src directory $startdir/src"
		exit 2
	fi
}

build() {
        cd $startdir
		mkdir -p pkg/usr/share/applications/
		mkdir -p pkg/usr/share/icons/hicolor/48x48/apps/
		cp nethack.desktop pkg/usr/share/applications/
		cp nethack.png pkg/usr/share/icons/hicolor/48x48/apps/
		cd src/$pkgname-$pkgver/sys/unix
        chmod +x setup.sh
        chmod +x nethack.sh
        ./setup.sh
        cd ../..
        # set configuration for config.h
        # set file compressor to gzip
        sed -i.orig -e 's#/usr/bin/compress#/usr/bin/gzip#;s#\.Z#.gz#;s#Lempel-Ziv#gzip#' include/config.h
        # define DLB option
        sed -i.orig -e 's|/\*\ #define\ DLB\ \*/|#define DLB|' include/config.h
        # define AUTOPICKUP_EXCEPTIONS option
        sed -i.orig -e 's|/\*#define\ AUTOPICKUP_EXCEPTIONS\ \*/|#define AUTOPICKUP_EXCEPTIONS|' include/config.h
        
        #set configuration for unixconf.h
        sed -i.orig -e 's|/\*\ #define\ LINUX\ \*/|#define LINUX|;s|/\*\ #define\ TIMED_DELAY\ \*/|#define TIMED_DELAY|' include/unixconf.h
        
        #set configuration for Makefile.src
        sed -i.orig -e 's|^CFLAGS\ =|#CFLAGS =|;s|^LFLAGS\ =|#LFLAGS =|' src/Makefile #remove default options
        sed -i.orig -e 's|#\ CC\ =\ gcc$|CC = gcc|;s|#\ LFLAGS\ =\ -L/usr/X11R6/lib|LFLAGS = -L/usr/X11R6/lib|;s|#\ CFLAGS\ =\ -O2\ -fomit-frame-pointer\ -I../include|CFLAGS = -O2 -march=i486 -mtune=i686 -fomit-frame-pointer -I../include|' src/Makefile
		# add more settings here for X11/gnome versions
		sed -i.orig -e 's|WINTTYLIB = -ltermlib|WINTTYLIB = -lncurses|' src/Makefile
		
        #set configuration for Makefile.utl
        sed -i.orig -e 's|^CFLAGS\ =|#CFLAGS =|;s|^LFLAGS\ =|#LFLAGS =|' util/Makefile
        sed -i.orig -e 's|#\ CC\ =\ gcc$|CC = gcc|;s|#\ LFLAGS\ =\ -L/usr/X11R6/lib|LFLAGS = -L/usr/X11R6/lib|;s|#\ CFLAGS\ =\ -O2\ -fomit-frame-pointer\ -I../include|CFLAGS = -O2 -march=i486 -mtune=i686 -fomit-frame-pointer -I../include|' util/Makefile
        sed -i.orig -e 's|^YACC\ \ \ \ \ =\ yacc|YACC     = bison -y|;s|LEX\ \ \ \ \ \ =\ lex|LEX      = flex|' util/Makefile
        
        #set configuration for Makefile.doc
        sed -i.orig -e "s|/usr/man/man6|${startdir}/pkg/usr/man/man6|" doc/Makefile
        
        # change location for default options file (quick-n-dirty)
        sed -i.orig -e 's|Sprintf(tmp_config,\ \"%s/%s\",\ envp,\ configfile|Sprintf(tmp_config, "/etc/nethack"|' src/files.c

		# create missing output folders
		mkdir -p ../../pkg/usr/man/man6
        mkdir -p ../../pkg/etc
        # create the default options file
        cat <<"EOF" >"../../pkg/etc/nethack"
# *** OPTIONS ***
#
# Visual options
OPTIONS=eight_bit_tty,colour
OPTIONS=hilite_pet
# game preferences
OPTIONS=autopickup,pickup_types:$"=/!?+
# interface options
OPTIONS=time,number_pad:0,lit_corridor,silent
# misc
OPTIONS=suppress_alert:3.3.1,!toptenwin


EOF
        # end of default options file
        #compile source
        cd src
        make
        cd ..
        #compile manpages
        cd doc
        make manpages
        cd ..
        
        #set configuration for Makefile.top
        sed -i.orig -e 's|GAMEUID\ \ =\ games|GAMEUID  = root|;s|GAMEGRP\ \ =\ bin|GAMEGRP  = games|;s|GAMEPERM\ =\ 04755|GAMEPERM = 02755|;s|FILEPERM\ =\ 0644|FILEPERM = 0664|;s|DIRPERM\ \ =\ 0755|DIRPERM  = 0775|' Makefile
        sed -i.orig -e "s|PREFIX	 = /usr|PREFIX	 = ${startdir}/pkg/usr|" Makefile
        sed -i.orig -e 's|SHELLDIR\ =\ \$(PREFIX)/games|SHELLDIR = $(PREFIX)/bin|' Makefile
        sed -i.orig -e "s|sed -e 's;/usr/games/lib/nethackdir;\$(GAMEDIR);'|sed |" Makefile
		
        # add more settings here for X11/gnome versions
        
        #build the remaining part & install
        make all
        make install
}

create_slackdesc() {
mkdir $startdir/pkg/install
cat <<"EODESC" >$startdir/pkg/install/slack-desc
nethack: nethack - single player dungeon exploration game
nethack: 
nethack: NetHack is an old but extremely rich (and hard) dungeon-hacking game.
nethack: Players can start with a variety of roles ranging from Barbarian to
nethack: Tourist, races and alignment. 
nethack: Your goal in the game is basically to go down to the end of the
nethack: dungeon, get the fabulous Amulet of Yendor and get out to offer it to
nethack: your God. You already know it won't be that easy...
nethack:  see also: http://nethack.org/
nethack: 
nethack: 
EODESC
}

copy_docs() {
	for stuff in ${docs[@]}; do
		if [ ! -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
			mkdir -p $startdir/pkg/usr/doc/$pkgname-$pkgver
		fi
		find $startdir/src -type f -iname "$stuff" -exec cp -R '{}' $startdir/pkg/usr/doc/$pkgname-$pkgver \;
	done
}
detect_dependencies() {
	echo "Detecting dependencies..."
	depends=$(requiredbuilder -z $startdir/$package.tgz)
	
	depstring=$depends
	unset FIRST
	for deps in $(echo "${depstring}" | sed 's/,/\n/g' | sort -u); do
		if [[ ! "$FIRST" ]]; then
			finaldeps=$deps,
			FIRST='1'
			continue
		fi
		finaldeps=${finaldeps}${deps},
	done
	echo "$finaldeps" | sed -e 's/^,//' -e 's/,$//' > $startdir/$package.dep
}
create_source_file(){
	[ -f $package.src ] && rm $package.src
	if [ ! -z $sourcetemplate ]; then
		echo $sourcetemplate/ZENBUILD >> $package.src
		echo $sourcetemplate/build-$pkgname.sh >> $package.src
	else
		echo ZENBUILD >> $package.src
		echo build-$pkgname.sh >> $package.src
	fi
	for SOURCES in ${source[@]}; do
		protocol=$(echo $SOURCES | sed 's|:.*||')
		if ! [ "$protocol" = "http" -o "$protocol" = "https" -o "$protocol" = "ftp" ]; then
			if [ ! -z $sourcetemplate ]; then
				echo $sourcetemplate/$(basename $SOURCES) >> $package.src
			else
				echo $(basename $SOURCES) >> $package.src
			fi
		else
			echo $SOURCES >> $package.src
		fi
	done
}
post_checks(){
	# Ideas taken from src2pkg :)
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		for DIRS in usr/doc/$pkgname-$pkgver usr/doc; do
			cd $startdir/pkg/$DIRS
			if [[ $(find . -type f) = "" ]] ; then
				cd ..
				rmdir $DIRS
			fi
		done
	fi
	# if the docs weren't deleted ...
	if [ -d "$startdir/pkg/usr/doc/$pkgname-$pkgver" ]; then
		cd $startdir/pkg/usr/doc/$pkgname-$pkgver
		#remove zero lenght files
		if [[ $(find . -type f -size 0) ]]; then
			echo "Removing some zero lenght files"
			find . -type f -size 0 -exec rm -f {} \;
		fi
	fi
	# check if we need to add code to handle info pages
	if [[ -d $startdir/pkg/usr/info ]] && [[ ! $(grep install-info $startdir/pkg/install/doinst.sh &> /dev/null) ]] ; then
		echo "Found info files - Adding install-info command to doinst.sh"
		INFO_LIST=$(ls -1 $startdir/pkg/usr/info)
		echo "" >> $startdir/pkg/install/doinst.sh
		echo "if [ -x usr/bin/install-info ] ; then" >> $startdir/pkg/install/doinst.sh
		for page in $(echo $INFO_LIST) ; do
			echo " usr/bin/install-info --info-dir=usr/info usr/info/$page 2>/dev/null" >> $startdir/pkg/install/doinst.sh
		done
		echo "fi" >> $startdir/pkg/install/doinst.sh
	fi
	[[ -e $startdir/pkg/usr/info/dir ]] && rm -f $startdir/pkg/usr/info/dir

	if [ -d $startdir/pkg/etc ]; then
		cd $startdir/pkg/
		for conf in $(find ./etc -type f) ; do
			conf=${conf: 2}
			dotnew=( "${dotnew[@]}" "$conf" )
		done
	fi
	if [[ "$dotnew" ]]; then
        for files in ${dotnew[@]} ; do
                fullfile="${startdir}/pkg/${files}"
                if [ -e "$fullfile" ]; then
                        mv $fullfile ${fullfile}.new
                else
                        echo "$fullfile was not found"
                        exit 2
                fi
        done
        cat<<"EODOTNEW" >>$startdir/pkg/install/doinst.sh
#Added by buildpkg 1.6.1
dotnew() {
        NEW="${1}.new"
        OLD="$1"
        if [ ! -e $OLD ]; then
                mv $NEW $OLD
        elif [ "$(cat $OLD | md5sum)" = "$(cat $NEW | md5sum)" ]; then
                rm $NEW
        fi
}
EODOTNEW
for i in ${dotnew[@]}; do
echo "dotnew $i" >> $startdir/pkg/install/doinst.sh
done
fi
}

####End Non Redundant Code############################

#Variables

startdir=$(pwd)
SRC=$startdir/src
PKG=$startdir/pkg

pkgname=nethack
pkgver=3.4.3
pkgrel=1
zenver=54
arch=i486
package=$pkgname-$pkgver-$arch-$zenver.$pkgrel
source=("ftp://heanet.dl.sourceforge.net/pub/sourceforge/n/ne/nethack/nethack-343-src.tgz" "nethack.desktop" "nethack.png")
sourcetemplate=http://durmont.free.fr/nethack
docs=(AUTHORS ChangeLog COPYING NEWS README TODO)
export CFLAGS="-O2 -march=i486 -mtune=i686"
export CXXFLAGS="-O2 -march=i486 -mtune=i686"

#Execution

check_for_root
clean_old_builds
prepare_directory
extract_source
set_pre_permissions
build
if [ ! "$?" = "0" ]; then
	echo "build() failed."
	exit 2
fi
create_slackdesc
post_checks
copy_docs
check_for_menu_compliance
strip_binaries
gzip_man_and_info_pages
set_post_permissions
copy_build_script
create_package
create_source_file
detect_dependencies
echo "Package has been built."
